from pyramid.config import Configurator
from charmworld.resources import Root
from charmworld.globals import default_globals_factory


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    config = Configurator(root_factory=Root, settings=settings)

#    config.add_view('charmworld.views.home',
#                    context='charmworld:resources.Root',
#                    renderer='charmworld:templates/index.pt')

    # Template globals
    config.set_renderer_globals_factory(default_globals_factory)

    config.add_static_view('static', 'charmworld:static', cache_max_age=3600)

    config.add_route(
        "json-charm-change", "/reports/charm-activity.json")
    config.add_route(
        "json-contributor-growth", "/reports/charm-contributors.json")
    config.add_route(
        "json-charm-growth", "/reports/charm-count.json")

    config.add_route("reports", "/reports")

    config.add_route("charmers", "/charmers")

    # Distro/Curated Charms
    config.add_route("charm-collection", "/charms")
    config.add_route("series-collection", "/charms/{series}")
    config.add_route("charm", "/charms/{series}/{charm}")
    config.add_route("charm-changes", "/charms/{series}/{charm}/changes")
    config.add_route("hook", "/charms/{series}/{charm}/hooks/{hook}")
    config.add_route("config", "/charms/{series}/{charm}/config")

    # Personal Charms
    config.add_route("personal-collection", "/~{owner}")
    config.add_route("personal-series", "/~{owner}/{series}")
    config.add_route("personal-charm", "/~{owner}/{series}/{charm}")
    config.add_route(
        "personal-charm-changes", "/~{owner}/{series}/{charm}/changes")
    config.add_route(
        "personal-hook", "/~{owner}/{series}/{charm}/hooks/{hook}")
    config.add_route(
        "personal-config", "/~{owner}/{series}/{charm}/config")

    config.add_route("interface", "/interfaces/{interface}")
    config.add_route("interface-collection", "/interfaces")
    config.add_route("interface-collection-map", "/interface-charm-map")


    # Tools
    config.add_route("tools-core-review", "/tools/core-review-queue")
    config.add_route("tools-review", "/review-queue")
    config.add_route("tools-store-missing", "/tools/store-missing")
    config.add_route("tools-maintainer-missing", "/tools/maintainer-missing")
    config.add_route("tools-proof-errors", "/tools/proof-errors")

    # Experiment
    config.add_route("charm-graph", "/charm-graph")
    config.add_route("json-interface-graph", "/interface-json-graph")

    # Feeds
    config.add_route("recently-changed", "/recently-changed")
    config.add_route("recently-changed-feed", "/feed/charm-changes")

    # Search
    config.add_route("search", "/search")

    # Home page
    config.add_route("home", "/")

    config.scan("charmworld")

    return config.make_wsgi_app()
