import json
import logging
import os
import redis
import subprocess
import shutil

from bzrlib.transport import get_transport
from bzrlib.branch import Branch

from routes import BZR_IN_QUEUE, BZR_OUT_QUEUE

ROOT_DIR = os.path.expanduser("/var/lib/charms")

log = logging.getLogger("charm.bzr")


def fetch_branch(root_dir, charm_data, retry=True):
    """Fetch a branch from bzr, and augment charm data.
    """
    branch_dir = os.path.abspath(
        str(os.path.join(root_dir,
                         charm_data["series"],
                         charm_data["owner"],
                         charm_data["name"],
                         charm_data["bname"])))

    if not os.path.exists(os.path.dirname(branch_dir)):
        os.makedirs(os.path.dirname(branch_dir))

    # Store the branch directory
    charm_data["branch_dir"] = branch_dir

    if not os.path.exists(branch_dir):
        log.info("Branching charm lp:%s", charm_data["branch_spec"])
        subprocess.check_output(
            ["/usr/bin/bzr", "co", "-q",
             "lp:%s" % charm_data["branch_spec"], branch_dir])
        return
    else:
        log.debug("Existing charm from lp:%s", charm_data["branch_spec"])
        transport = get_transport(branch_dir)
        branch = Branch.open_from_transport(transport)
        cur_rev_id = branch.last_revision()
        if cur_rev_id == charm_data['commit']:
            log.debug("Already up to date lp:%s", charm_data["branch_spec"])
            return

    log.debug("Updating branch lp:%s", charm_data["branch_spec"])

    try:
        subprocess.check_output(
            ["/usr/bin/bzr", "update", "-q"],
            cwd=branch_dir,
            stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError:
        if retry:


            shutil.rmtree(branch_dir)
            return fetch_branch(root_dir, charm_data, retry=False)
        raise


def fetch_branches(server,
                   root_dir,
                   queue=BZR_IN_QUEUE,
                   scan_queue=BZR_OUT_QUEUE):
    """
    """
    while 1:
        item = server.spop(queue)
        if not item:
            return
        try:
            charm_data = json.loads(item)
            fetch_branch(root_dir, charm_data)
        except Exception, e:
            log.exception("bzr error error on %s: %s", item, str(e))
            charm_data["error_stage"] = "bzr"
            if hasattr(e, 'output'):
                charm_data["error"] = str(e.output)
            continue
        else:
            server.lpush(scan_queue, json.dumps(charm_data))


def main():
    logging.basicConfig(
        level=logging.WARNING,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    server = redis.Redis()
    fetch_branches(server, ROOT_DIR)


if __name__ == '__main__':
    main()
