"""
Juju Core Review

 Contributed by Juan Negronil


 Changes
 - convert to job.
 - use dateutil date parsing instead of string comparsion.
 - merge proposals show target with series/branch.

"""

from launchpadlib.launchpad import Launchpad
from dateutil.parser import parse

import itertools
import pymongo
import logging
import time

log = logging.getLogger("core.review")

def update_review_queue(db):
    st = time.time()
    lp = Launchpad.login_anonymously('charm-tools', 'production', version='devel')

    # Entity roots for items of interests
    #charm = lp.distributions['charms']
    #charmers = lp.people['charmers']
    #charm_contributors = lp.people['charm-contributors']

    log.debug("lp login time %0.2f", time.time()-st)

    st = time.time()
    #bugs = charm.searchTasks(
    #    tags=['new-formula', 'new-charm'],
    #    tags_combinator="Any",
    #    status=['New', 'Confirmed', 'Triaged', 'In Progress', 'Fix Committed'])
    #charmers_bugs = charmers.searchTasks(
    #    status=['New', 'Confirmed', 'Triaged', 'In Progress', 'Fix Committed'])
    #proposals = charmers.getRequestedReviews(status="Needs review")
    #contributor_proposals = charm_contributors.getRequestedReviews(status="Needs review")

    juju = lp.projects['juju']
    branch_proposals = [mp for mp in juju.getMergeProposals().entries \
                     if mp['target_branch_link'].endswith("~juju/juju/trunk")]
    branch_proposals = [mp for mp in branch_proposals if not mp['queue_status'] in (
        'Work in progress',)]
    log.debug("lp query time %0.2f", time.time()-st)

    r_queue = list()
    #for bug in itertools.chain(bugs.entries, charmers_bugs.entries):
    #    entry = {
    #        'date_created' : parse(bug['date_created']),
    #        'summary' : bug['title'].split('"')[1].strip(),
    #        'item' : bug['web_link'],
    #        'status' : bug['status'],
    #        }
    #    r_queue.append(entry)

    for proposal in branch_proposals:
        #proposals.entries, contributor_proposals.entries, doc_proposals):

        parts = proposal['target_branch_link'].rsplit('/', 3)
        if parts[-1] == "trunk":
            parts.pop(-1)

        target = "/".join(parts[2:])
        origin =  "/".join(proposal['source_branch_link'].rsplit('/', 3)[1:])

        entry = {
            'date_created' : parse(proposal['date_created']),
            'summary' : "Merge for %s from %s" % (target, origin),
            'item' : proposal['web_link'],
            'status' : proposal['queue_status'],
            }
        r_queue.append(entry)

    # Drop old collection
    log.info("Dropping old review queue (%d items)",
             db['core_review_queue'].count())
    db.drop_collection('core_review_queue')

    # Bulk insert
    log.info("Updating review queue (%d items)" % len(r_queue))
    db['core_review_queue'].insert(r_queue)


def main():
    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    c = pymongo.Connection()
    db = c['juju']
    update_review_queue(db)

if __name__ == '__main__':
    main()
