import urllib
import json
import redis
import logging
import pymongo

from utils import parse_branch
from routes import LP_OUT_QUEUE

BRANCH_TIPS = "https://api.launchpad.net/devel/charm?ws.op=getBranchTips"
CHARM_DIR = ""

log = logging.getLogger("charm.launchpad")


def available_charms():
    charm_data = urllib.urlopen(BRANCH_TIPS).read()
    charm_data = json.loads(charm_data)

    #charm_data = json.loads(open("charms.json").read())

#    charms = {}

    for repo, commit, series in charm_data:
        _, branch_name = repo.rsplit("/", 1)

        data = parse_branch(repo, series, commit)
        if not data:
            log.warning("Unable to parse repo %s", repo)
            continue

#        if not repo_owner in ("~charmers",):
#            log.debug("Skipped r-O %s", repo)
#            continue

        if data["bname"] not in ("trunk", "trunk-1"):
            #log.debug("Skipped branch %s", repo)
            continue
        yield data

 #       if not series:
 #           log.debug("Skipped r-S %s", repo)
 #           continue

        # Prefer trunk-1 over trunk (has series), need to keep in mem
        # to filter this.
#        if data["name"] in charms:
#            if data["bname"] == "trunk-1":
#                charms[data["name"]] = data
#        else:
#            charms[data["name"]] = data




def queue_charms(server, db):
    for charm in available_charms():

#        record = db.charms.find_one(
#            {"branch_spec": charm['branch_spec']})
#        if record is not None:
#            if record['commit'] == charm['commit']:
#                continue

        added = server.sadd(LP_OUT_QUEUE, json.dumps(charm))
        if added and 0:
            log.info("Queued %s", charm)


def main():
    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    server = redis.Redis()
    db = pymongo.Connection()['juju']
    queue_charms(server, db)


if __name__ == '__main__':
    main()
