

import json
import logging
import redis
import subprocess
import os


from routes import PROOF_IN_QUEUE, PROOF_OUT_QUEUE

CHARM_PROOF_BIN = "~/tools/charm-tools/scripts/proof"

log = logging.getLogger("charm.proof")


def proof_charm(charm):
    try:
        output = subprocess.check_output(
            [os.path.expanduser(CHARM_PROOF_BIN),
             charm['branch_dir']],
            stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError, e:
        if not e.returncode in (-1, 100, 200):
            raise
        output = e.output

    proof = {}
    for l in output.strip().split("\n"):
        if not ':' in l:
            continue
        level, msg = l.split(':', 1)
        if level == "W" and 'name' in msg:
            continue
        proof.setdefault(level.lower(), []).append(msg)

    charm['proof'] = proof


def run(server, in_queue=PROOF_IN_QUEUE, out_queue=PROOF_OUT_QUEUE):
    while 1:
        item = server.lpop(in_queue)
        if not item:
            return

        try:
            charm_data = json.loads(item)
            log.info("Proofing charm %s", charm_data["branch_spec"])
            proof_charm(charm_data)
        except subprocess.CalledProcessError, e:
            log.warning("proof error on %s: %s", charm_data['branch_spec'], str(e))
            charm_data['proof'] = {'p': e.output}
            charm_data['proof'].setdefault('c', []).append('raised runtime error in proof')
            continue
        except Exception, e:
            log.exception("proof error on %s: %s", charm_data['branch_spec'], str(e))
            continue
        else:
            server.lpush(out_queue, json.dumps(charm_data))


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    server = redis.Redis()
    run(server)
