from datetime import datetime
import json
import logging
#import os
import redis
from urllib2 import urlopen

from routes import STORE_IN_QUEUE, STORE_OUT_QUEUE

log = logging.getLogger("charm.store")


STORE_URL = "https://store.juju.ubuntu.com"


def check_store(charm, retry_address=False):

    if retry_address:
        address = retry_address
    if charm['owner'] == 'charmers':
        address = "cs:%s/%s" % (charm["series"], charm["name"])
    else:
        address = "cs:~%s/%s/%s" % (
            charm["owner"], charm["series"], charm["name"])


    def _store_get(address):
        url = STORE_URL + "/charm-info?charms=%s&stats=0" % address
        contents = urlopen(url).read()
        data = json.loads(contents)
        data = data[address]
        data['address'] = address
        return data

    data =_store_get(address)

    if data.get("errors") or data.get('warnings'):
        log.warning("not found %s" % data)
#        if charm['owner'] == 'charmers' and not retry_address:
#            check_store(
#                charm, "cs:~%s/%s/%s" % (
#                    charm["owner"], charm["series"], charm["name"]))

    if not 'store_data' in charm:
        data["store_checked"] = datetime.now().ctime()
        charm['store_data'] = data


def run(server,
        in_queue=STORE_IN_QUEUE,
        out_queue=STORE_OUT_QUEUE):

    while 1:
        #item = server.lpop(in_queue)
        item = server.lpop(in_queue)
        if not item:
            return

        try:
            charm_data = json.loads(item)
            log.info("Checking store for %s", charm_data["branch_spec"])
            check_store(charm_data)
        except Exception, e:
            log.exception("store error on %s: %s", item, str(e))
            continue
        else:
            server.lpush(out_queue, json.dumps(charm_data))


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    server = redis.Redis()
    run(server)
