

def parse_branch(branch_spec, series=(), commit=None):
    """Parse a branch spec into a dictionary

    A branch spec looks like the following:

      ~<owner>/charm/<series>/<charm-name>/<branch-name>

    The dictionary has the following.
      -owner
      -series
      -name
      -bname

    Returns none if unable to parse.

    Returned data is augmented with commit if present
    """

    data = {}

    parts = branch_spec.split("/")

    if not len(parts) == 5:
        raise ValueError("Invalid branch spec %s" % branch_spec)

    data["owner"] = parts[0][1:]
    data["series"] = parts[2]
    data["name"] = parts[3]
    data["bname"] = parts[4]
    data["branch_spec"] = branch_spec

    if series:
        data["series"] = series[0]

    if commit:
        data["commit"] = commit

    return data

# Testing

BRANCH_TIPS = "https://api.launchpad.net/devel/charm?ws.op=getBranchTips"


def test():
    import urllib
    import json
    data = urllib.urlopen(BRANCH_TIPS).read()
    data = json.loads(data)

    for repo, commit, series in data:
        print repo, parse_branch(repo)

if __name__ == '__main__':
    test()
